@@ -27,9 +27,11 @@
 #include <string.h>
 #include <ctype.h>
 #include "xalloc.h"
+#include "xvasprintf.h"
+#include "memcmp2.h"
+#include "minmax.h"
 
 #include "astr.h"
-#include "xalloc_extra.h"
 
 #define ALLOCATION_CHUNK_SIZE	16
 
@@ -174,6 +176,12 @@ astr_substr (astr as, size_t pos, size_t size)
   return astr_ncat_cstr (astr_new (), astr_cstr (as) + pos, size);
 }
 
+int
+astr_cmp (astr as1, astr as2)
+{
+  return memcmp2 (astr_cstr (as1), astr_len (as1), astr_cstr (as2), astr_len (as2));
+}
+
 astr
 astr_replace_cstr (astr as, size_t pos, size_t size, const char *s)
 {
@@ -215,11 +223,11 @@ astr
 astr_afmt (astr as, const char *fmt, ...)
 {
   va_list ap;
-  char *buf;
+  const char *buf;
   va_start (ap, fmt);
-  xvasprintf (&buf, fmt, ap);
+  buf = xvasprintf (fmt, ap);
   astr_cat_cstr (as, buf);
-  free (buf);
+  free ((char *) buf);
   va_end (ap);
   return as;
 }
@@ -231,7 +239,6 @@ astr
 astr_recase (astr as, enum casing newcase)
 {
   astr bs = astr_new ();
-  int (*func) (int) = NULL;
   size_t i, len;
 
   if (newcase == case_capitalized || newcase == case_upper)
@@ -239,20 +246,8 @@ astr_recase (astr as, enum casing newcase)
   else
     astr_cat_char (bs, tolower (astr_get (as, 0)));
 
-  switch (newcase)
-    {
-    case case_upper:
-      func = toupper;
-      break;
-    case case_lower:
-      func = tolower;
-      break;
-    default:
-      break;
-    }
-
   for (i = 1, len = astr_len (as); i < len; i++)
-    astr_cat_char (bs, func ? func (astr_get (as, i)) : astr_get (as, i));
+    astr_cat_char (bs, ((newcase == case_upper) ? toupper : tolower) (astr_get (as, i)));
 
   astr_cpy (as, bs);
   astr_delete (bs);
