@@ -236,7 +236,7 @@ Just C-u as argument means to use the current column.
 {
   long fill_col = (lastflag & FLAG_UNIARG_EMPTY) ?
     get_buffer_pt (cur_bp).o : (unsigned long) uniarg;
-  char *buf;
+  char *buf = NULL;
 
   if (!(lastflag & FLAG_SET_UNIARG) && arglist == NULL)
     {
@@ -259,7 +259,7 @@ Just C-u as argument means to use the current column.
     }
   else
     {
-      xasprintf (&buf, "%ld", fill_col);
+      buf = xasprintf ("%ld", fill_col);
       /* Only print message when run interactively. */
       minibuf_write ("Fill column set to %s (was %d)", buf,
                      get_variable_number ("fill-column"));
@@ -472,42 +472,40 @@ END_DEFUN
 /*
  * Compact the spaces into tabulations according to the `tw' tab width.
  */
-static void
-tabify_string (char *dest, char *src, size_t scol, size_t tw)
+static astr
+tabify_string (astr src, size_t scol, size_t tw)
 {
-  char *sp, *dp;
-  size_t dcol = scol, ocol = scol;
+  astr dest = astr_new ();
+  size_t dcol = scol, i;
 
-  for (sp = src, dp = dest;; ++sp)
-    switch (*sp)
-      {
-      case ' ':
-        ++dcol;
-        break;
-      case '\t':
-        dcol += tw;
-        dcol -= dcol % tw;
-        break;
-      default:
-        while (((ocol + tw) - (ocol + tw) % tw) <= dcol)
-          {
-            if (ocol + 1 == dcol)
-              break;
-            *dp++ = '\t';
-            ocol += tw;
-            ocol -= ocol % tw;
-          }
-        while (ocol < dcol)
-          {
-            *dp++ = ' ';
-            ocol++;
-          }
-        *dp++ = *sp;
-        if (*sp == '\0')
-          return;
-        ++ocol;
-        ++dcol;
-      }
+  for (i = 0; i < astr_len (src); i++)
+    {
+      char c = astr_get (src, i);
+      switch (c)
+        {
+        case ' ':
+          ++dcol;
+          break;
+        case '\t':
+          dcol += tw;
+          dcol -= dcol % tw;
+          break;
+        default:
+          while (((scol + tw) - (scol + tw) % tw) <= dcol && scol != dcol - 1)
+            {
+              astr_cat_char (dest, '\t');
+              scol += tw;
+              scol -= scol % tw;
+            }
+          for (; scol < dcol; scol++)
+            astr_cat_char (dest, ' ');
+          astr_cat_char (dest, c);
+          ++scol;
+          ++dcol;
+        }
+    }
+
+  return dest;
 }
 
 /*
@@ -515,40 +513,35 @@ tabify_string (char *dest, char *src, size_t scol, size_t tw)
  * The output buffer should be big enough to contain the expanded string,
  * i.e. strlen (src) * tw + 1.
  */
-static void
-untabify_string (char *dest, char *src, size_t scol, size_t tw)
+static astr
+untabify_string (astr src, size_t scol, size_t tw)
 {
-  char *sp, *dp;
-  int col = scol;
+  astr dest = astr_new ();
+  size_t i;
 
-  for (sp = src, dp = dest; *sp != '\0'; ++sp)
-    if (*sp == '\t')
-      {
+  for (i = 0; i < astr_len (src); i++)
+    {
+      char c = astr_get (src, i);
+      if (c == '\t')
         do
-          *dp++ = ' ', ++col;
-        while ((col % tw) > 0);
-      }
-    else
-      *dp++ = *sp, ++col;
-  *dp = '\0';
+          astr_cat_char (dest, ' ');
+        while ((++scol % tw) > 0);
+      else
+        {
+          astr_cat_char (dest, c);
+          ++scol;
+        }
+    }
+
+  return dest;
 }
 
-#define TAB_TABIFY	1
-#define TAB_UNTABIFY	2
 static void
 edit_tab_line (Line * lp, size_t lineno, size_t offset, size_t size,
-               int action)
+               astr (*action) (astr as, size_t scol, size_t tw))
 {
-  char *src, *dest;
   size_t col, i, t = tab_width (cur_bp);
-
-  if (size == 0)
-    return;
-
-  src = (char *) xzalloc (size + 1);
-  dest = (char *) xzalloc (size * t + 1);
-  strncpy (src, astr_cstr (get_line_text (lp)) + offset, size);
-  src[size] = '\0';
+  astr src = astr_substr (get_line_text (lp), offset, size), dest;
 
   /* Get offset's column.  */
   col = 0;
@@ -559,25 +552,22 @@ edit_tab_line (Line * lp, size_t lineno, size_t offset, size_t size,
       ++col;
     }
 
-  /* Call un/tabify function.  */
-  if (action == TAB_UNTABIFY)
-    untabify_string (dest, src, col, t);
-  else
-    tabify_string (dest, src, col, t);
+  dest = action (src, col, t);
 
-  if (strcmp (src, dest) != 0)
+  /* Only make an edit if the line has changed. */
+  if (astr_cmp (src, dest) != 0)
     {
       undo_save (UNDO_REPLACE_BLOCK, make_point (lineno, offset),
-                 size, strlen (dest));
-      line_replace_text (lp, offset, size, dest, false);
+                 size, astr_len (dest));
+      line_replace_text (lp, offset, size, astr_cstr (dest), false);
     }
 
-  free (src);
-  free (dest);
+  astr_delete (src);
+  astr_delete (dest);
 }
 
 static le *
-edit_tab_region (int action)
+edit_tab_region (astr (*action) (astr as, size_t scol, size_t tw))
 {
   Region * rp = region_new ();
 
@@ -599,7 +589,7 @@ edit_tab_region (int action)
           /* First line.  */
           if (lineno == get_region_start (rp).n)
             {
-              /* Region on a sole line. */
+              /* Region on a single line. */
               if (lineno == get_region_end (rp).n)
                 edit_tab_line (lp, lineno, get_region_start (rp).o, get_region_size (rp), action);
               /* Region is multi-line. */
@@ -632,10 +622,10 @@ DEFUN ("tabify", tabify)
 Convert multiple spaces in region to tabs when possible.
 A group of spaces is partially replaced by tabs
 when this can be done without changing the column they end at.
-The variable `tab-width' controls the spacing of tab stops.
+The variable @samp{tab-width} controls the spacing of tab stops.
 +*/
 {
-  ok = edit_tab_region (TAB_TABIFY);
+  ok = edit_tab_region (tabify_string);
 }
 END_DEFUN
 
@@ -645,7 +635,7 @@ Convert all tabs in region to multiple spaces, preserving columns.
 The variable @samp{tab-width} controls the spacing of tab stops.
 +*/
 {
-  ok = edit_tab_region (TAB_UNTABIFY);
+  ok = edit_tab_region (untabify_string);
 }
 END_DEFUN
 
@@ -787,9 +777,9 @@ move_sexp (int dir)
               level++;
               gotsexp = true;
               if (c == '\"')
-                double_quote ^= 1;
+                double_quote = !double_quote;
               if (c == '\'')
-                single_quote ^= 1;
+                single_quote = !double_quote;
             }
           else if (dir > 0 ? ISCLOSEBRACKETCHAR (c) : ISOPENBRACKETCHAR (c))
             {
@@ -799,9 +789,9 @@ move_sexp (int dir)
               level--;
               gotsexp = true;
               if (c == '\"')
-                double_quote ^= 1;
+                double_quote = !double_quote;
               if (c == '\'')
-                single_quote ^= 1;
+                single_quote = !single_quote;
 
               if (level < 0)
                 {
@@ -1320,12 +1310,11 @@ END_DEFUN
  * Set the region case.
  */
 static le *
-setcase_region (enum casing rcase)
+setcase_region (int (*func) (int))
 {
   Region * rp = region_new ();
   Line * lp;
   size_t i, size;
-  int (*func) (int) = rcase == case_upper ? toupper : tolower;
 
   if (warn_if_readonly_buffer () || !calculate_the_region (rp))
     {
@@ -1364,7 +1353,7 @@ DEFUN ("upcase-region", upcase_region)
 Convert the region to upper case.
 +*/
 {
-  ok = setcase_region (case_upper);
+  ok = setcase_region (toupper);
 }
 END_DEFUN
 
@@ -1373,7 +1362,7 @@ DEFUN ("downcase-region", downcase_region)
 Convert the region to lower case.
 +*/
 {
-  ok = setcase_region (case_lower);
+  ok = setcase_region (tolower);
 }
 END_DEFUN
 
@@ -1393,7 +1382,7 @@ pipe_command (const char *cmd, const char *tempfile, bool insert, bool replace)
   char *cmdline, *eol;
   FILE * pipe;
 
-  xasprintf (&cmdline, "%s 2>&1 <%s", cmd, tempfile);
+  cmdline = xasprintf ("%s 2>&1 <%s", cmd, tempfile);
   pipe = popen (cmdline, "r");
   free (cmdline);
   if (pipe == NULL)
@@ -1599,11 +1588,8 @@ On nonblank line, delete any immediately following blank lines.
           activate_mark ();
           while (FUNCALL (forward_line) == leT && is_blank_line ())
             ;
-          if (!seq_started)
-            {
-              seq_started = true;
-              undo_save (UNDO_START_SEQUENCE, get_marker_pt (m), 0, 0);
-            }
+          seq_started = true;
+          undo_save (UNDO_START_SEQUENCE, get_marker_pt (m), 0, 0);
           FUNCALL (delete_region);
           pop_mark ();
         }
